/*-
 * Copyright (c) 2000 Edward Brocklesby.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/* parse a configuration file */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "getusers.h"

struct config_s config;

struct {
	char *option;
	enum {STRING, INT, YESNO} type;
	void *value;
} conf_tab[] = {
	{"server", STRING, &config.server},
	{"port",   INT,    &config.port},
	{"vhost",  STRING, &config.vhost},
	{"target", STRING, &config.target},
	{"mrtg",   YESNO,  &config.mrtg},
	{"opernick", STRING, &config.opernick},
	{"operpass", STRING, &config.operpass},
	{"oper", YESNO, &config.oper},
	{"localonly", YESNO, &config.lo},
	{"globalonly", YESNO, &config.go},
	{NULL,     STRING, NULL}
};

int
parse_config(char *conffile)
{
	char line[128];
	FILE *file;

	if((file = fopen(conffile, "r")) == NULL) 
		return -1;

	while(fgets(line, sizeof(line), file) != NULL) {
		int argc, i;
		char *argv[MAX_ARG];
		int matched = 0;
		
		line[strlen(line)-1] = '\0';

		if (line[0] == '#')
			continue;

		if (strlen(line) == 0)
			continue;

		create_raw_vector(&argc, argv, line);

		for (i = 0; conf_tab[i].option; i++) {
			if (!strcasecmp(conf_tab[i].option, argv[0])) {
				switch (conf_tab[i].type) {
					case STRING:
						*((char **)(conf_tab[i].value)) = malloc(strlen(argv[1]) + 1);
						strcpy(*((char **)(conf_tab[i].value)), argv[1]);
						break;
					case YESNO:
						if (!strcasecmp(argv[1], "yes")) 
							*(int *)(conf_tab[i].value) = 1;
						else
							*(int *)(conf_tab[i].value) = 0;
						break;
					case INT:
						*(int *)(conf_tab[i].value) = atoi(argv[1]);
						break;
				}
				matched = 1;
				break;
			}
		}
		if (!matched) {
			fprintf(stderr, "error  = unknown config option %s\n", argv[0]);
			exit(1);
		}
	}
	return 0;
}
