#include <sys/types.h>
#include <sys/socket.h>

#include <netinet/in.h>

#include <arpa/inet.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <unistd.h>
#include <stdarg.h>
#include <errno.h>

#include "getusers.h"

#ifndef MSG_NOSIGNAL
# define MSG_NOSIGNAL 0
#endif

char *
mknick(void)
{
	static char nick[10];
	int i;

	srandom(getpid());
	strcpy(nick, "_stat");

	for (i = 0; i < 4; i++) 
		nick[i + 5] = '0' + (random() % 9);		

	nick[9] = '\0';
	return nick;
}

int 
sock_connect(char *server, int port, char *vhost)
{
        struct sockaddr_in dest_addr;
	struct sockaddr_in src_addr;
        struct hostent *h;
	int sockfd;

        sockfd = socket(AF_INET, SOCK_STREAM, 0);
        if (sockfd < 0)
                return -1;

        h = gethostbyname(server);

	fprintf(stderr, " DNS");
	fflush(stderr);

        if (!h) {
                return -1;
        } else {
                bzero((char *) &dest_addr, sizeof(dest_addr));
                bcopy(h->h_addr, (char *) &dest_addr.sin_addr, h->h_length);
                dest_addr.sin_family = h->h_addrtype;
        }

	if (vhost) {
		h = gethostbyname(vhost);
		fprintf(stderr, " VDNS");

		bzero((char *) &src_addr, sizeof(src_addr));
		src_addr.sin_port = 0;
		src_addr.sin_family = AF_INET;
		bcopy(h->h_addr, (char *) &src_addr.sin_addr, h->h_length);

		if (bind(sockfd, (struct sockaddr *) &src_addr, sizeof(src_addr))) {
			fprintf(stderr, "\nerror  = %d\n", errno);
			exit(1);
		}
	}

        dest_addr.sin_port = (unsigned short) htons(port);
        if (connect (sockfd, (struct sockaddr *) &dest_addr, sizeof(struct sockaddr)) == -1)
                return -1;

        sock_send(sockfd, "NICK %s", mknick());
        sock_send(sockfd, "USER %s %s %s :Server Statistics Bot", getlogin(), getlogin(),
                  server);
        return sockfd;
}

void 
sock_send(int sockfd, char *sendtext, ...)
{
        int r1, r2;

        va_list args;
        char putbuf[1024];

        va_start(args, sendtext);
        vsnprintf(putbuf, 1024, sendtext, args);
        va_end(args);

        r1 = send(sockfd, putbuf, strlen(putbuf), MSG_NOSIGNAL);
        r2 = send(sockfd, "\n", 1, MSG_NOSIGNAL);

        if (r1 == -1 || r2 == -1) {
                fprintf(stderr, "\nerror  = %s\n", strerror(errno));
		exit(1);
        }
}

#define IRC_BUFSIZ 512

int 
irc_readbyte(int s)
{
        static char buffer[IRC_BUFSIZ];
        static int data_in_buffer = 0;
        static char *buf_p = NULL; /* pointer to current buffer */
        int i, c;

        /* nothing in the buffer, so get some data from the server */
        if (!data_in_buffer) {
                i = recv(s, &buffer, IRC_BUFSIZ, MSG_NOSIGNAL);

                if (i == -1) {
                        fprintf(stderr, "\nerror  = %s\n", strerror(errno));
                        exit(8);
                }

                data_in_buffer = i;
                buf_p = &buffer[0];
        }

        c = *buf_p;
        data_in_buffer--;
        buf_p++;

        return c;
}

char *
sock_read_line(int socket)
{
        char *message = NULL;
        int message_size = 0;
        char c;
        int i = 0;
        
        for (;;) {
                c = irc_readbyte(socket);

                message = realloc(message, message_size + 1);
                message[i] = c;
                message_size++;
                if(message[i] == '\n') {
                        message[i - 1] = 0;
                        return message;
                }
                i++;
        }
}

