/*-
 * Copyright (c) 2000 Dan Papasian.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  $Id: string.c,v 1.22 2000/07/19 02:07:30 bugg Exp $
 */

#include <sys/types.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "getusers.h"

/* Create an argument vector from the string, understanding longargs with colon */

int
create_vector(int *argc, char *argv[MAX_ARG], char *string) 
{
	char *longarg = NULL;
	int i = 0;

	*argc = 0;

	longarg = string;

	if(strsep(&longarg,":")) /* Tear off short args */
		if(longarg && !strcmp(longarg, string)) /* If there wasn't a longarg..*/
			longarg = NULL;

	if(longarg)
		string[(longarg-string)-1] = '\0';

	if(create_raw_vector(argc, argv, string)) {
		return -1;
	}

	i = *argc;

	if(longarg)
		argv[i] = strdup(longarg), ++i;

	*argc = i;
	argv[*argc] = NULL;
	return 0;
}

/* Same as above, but don't use longarg */
int
create_raw_vector(int *argc, char *argv[MAX_ARG], char *string) 
{
	char *ap;
	int i = 0;

	while((ap = strsep(&string, " ")) != NULL) {
		if(*ap != '\0') {
			argv[i] = strdup(ap);
			if(i < MAX_ARG)
				++i;
			else
				break;
		}
	}

	if(i == MAX_ARG) {
		free_vector(argv, i);
		return -1;
	}
	
	*argc = i;
	argv[*argc] = NULL;
	return 0;
}

void
free_vector(char **argv, int argc)
{
	while(argc)
		free(argv[argc--]);

	free(argv[0]);

}

#ifdef NO_SNPRINTF
int
snprintf(char *str, size_t size, char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	vsprintf(str, fmt, ap);
	va_end(ap);
}
#endif

#ifdef NO_ASPRINTF
int
asprintf(char **ret, char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	*ret = malloc(MAX_STRING);
	vsnprintf(*ret, MAX_STRING, fmt, ap);
	va_end(ap);
}
#endif

#ifdef NO_STRSEP
char *
strsep(char **stringp, char *delim)
{
        char *s;
        char *spanp;
        int c, sc;
        char *tok;

        if ((s = *stringp) == NULL)
                return NULL;
        for (tok = s;;) {
                c = *s++;
                do {
                        if ((sc = *spanp++) == c) {
                                if (c == 0)
                                        s = NULL;
                                else
                                        s[-1] = 0;
                                *stringp = s;
                                return tok;
                        }
                } while (sc != 0);
        }
	return NULL;
}
#endif
